@@ -37,51 +37,69 @@ relatedConcepts <- function(conceptIds,
 
 	active <- sourceId <- destinationId <- conceptId <- NULL
 
-	conceptIds <- as.SNOMEDconcept(conceptIds, SNOMED = SNOMED)
+	conceptIds <- unique(as.SNOMEDconcept(conceptIds, SNOMED = SNOMED))
+	typeId <- as.SNOMEDconcept(typeId, SNOMED = SNOMED)
 	
 	# If no concepts supplied, return an empty vector
 	if (length(conceptIds) == 0){
 		return(conceptIds)
 	}
-	
-	typeId <- as.SNOMEDconcept(typeId, SNOMED = SNOMED)
-	if (reverse){
-		TOLINK <- data.table(destinationId = conceptIds, typeId = typeId)
-	} else {
-		TOLINK <- data.table(sourceId = conceptIds, typeId = typeId)
-	}
-	OUT <- data.table(active = logical(0),
-		conceptId = bit64::integer64(0))
-	# Retrieve relationship table
-	addRelationship <- function(tablename, OUT){
-		TABLE <- get(tablename, envir = SNOMED)
+
+	if (active_only){
 		if (reverse){
-			OUT <- rbind(OUT, TABLE[TOLINK,
-				on = c('destinationId', 'typeId')][,
-				list(active, conceptId = sourceId)])
+			TOLINK <- data.table(destinationId = conceptIds,
+				typeId = typeId, active = TRUE)
 		} else {
-			OUT <- rbind(OUT, TABLE[TOLINK,
-				on = c('sourceId', 'typeId')][,
-				list(active, conceptId = destinationId)])
+			TOLINK <- data.table(sourceId = conceptIds,
+				typeId = typeId, active = TRUE)
+		}
+	} else {
+		if (reverse){
+			TOLINK <- data.table(destinationId = conceptIds,
+				typeId = typeId)
+		} else {
+			TOLINK <- data.table(sourceId = conceptIds,
+				typeId = typeId)
+		}	
+	}
+
+	# Retrieve concepts with the relevant relationship
+	getRelationship <- function(tablename, active_only){
+		TABLE <- get(tablename, envir = SNOMED)
+		if (active_only){
+			if (reverse){
+				return(TABLE[TOLINK, on = c('destinationId', 'typeId',
+					'active')]$sourceId)
+			} else {
+				return(TABLE[TOLINK, on = c('sourceId', 'typeId',
+					'active')]$destinationId)
+			}
+		} else {
+			if (reverse){
+				return(TABLE[TOLINK,
+					on = c('destinationId', 'typeId')]$sourceId)
+			} else {
+				return(TABLE[TOLINK,
+					on = c('sourceId', 'typeId')]$destinationId)
+			}
 		}
-		OUT
 	}
 
 	# Add relationships from each table
-	for (table in tables){
-		OUT <- addRelationship(table, OUT)
-	}
-
-	# Limit to active terms if required
-	if (active_only){
-		out <- OUT[active == TRUE]$conceptId
-	} else {
-		out <- OUT$conceptId
+	out <- unique(as.SNOMEDconcept(getRelationship(tables[1],
+		active_only), SNOMED = SNOMED))
+	if (length(tables) > 1){
+		for (table in tables[-1]){
+			out <- union(out, getRelationship(table, active_only))
+		}
 	}
+	# Remove NA values
+	out <- out[!is.na(out)]
 
 	# Recursion if appropriate
 	if (recursive == TRUE){
-		out <- sort(unique(c(conceptIds, out)))
+		# Include original concepts for recursion
+		out <- union(conceptIds, out)
 		if (length(conceptIds) < length(out)){
 			# Recurse
 			return(relatedConcepts(conceptIds = out,
@@ -89,10 +107,10 @@ relatedConcepts <- function(conceptIds,
 				reverse = reverse, recursive = TRUE,
 				active_only = active_only))
 		} else {
-			return(as.SNOMEDconcept(unique(out)))
+			return(out)
 		}
 	} else {
-		return(as.SNOMEDconcept(unique(out)))
+		return(out)
 	}
 }
 
@@ -246,7 +264,8 @@ createTransitive <- function(conceptIds, SNOMED = getSNOMED(),
 		
 	# Define symbols for R CMD check
 	childId <- parentId <- sourceId <- destinationId <- typeId <- NULL
-		
+	descendantId <- ancestorId <- NULL
+	
 	conceptIds <- as.SNOMEDconcept(conceptIds, SNOMED = SNOMED)
 	WORKING <- rbindlist(lapply(tables, function(x){
 		TEMP <- get(x, envir = SNOMED)
@@ -269,12 +288,14 @@ createTransitive <- function(conceptIds, SNOMED = getSNOMED(),
 			WORKING[, list(selfId = childId, parentId)], by = 'selfId',
 			allow.cartesian = TRUE)[, list(childId, parentId)])
 		WORKING <- WORKING[!duplicated(WORKING)]
-		gc()
 		old_nrows <- new_nrows
 		new_nrows <- nrow(WORKING)
 	}
-	WORKING[childId %in% conceptIds & parentId %in% conceptIds,
+	OUT <- WORKING[childId %in% conceptIds & parentId %in% conceptIds,
 		list(ancestorId = parentId, descendantId = childId)]
+	setkey(OUT, descendantId)
+	setindex(OUT, ancestorId)
+	OUT
 }
 
 #' Whether SNOMED CT concepts have particular attributes
@@ -296,6 +317,8 @@ createTransitive <- function(conceptIds, SNOMED = getSNOMED(),
 #' @return a vector of Booleans stating whether the attribute exists
 #' @export
 #' @examples
+#' data.table::setDTthreads(threads = 1) # for CRAN testing
+#'
 #' SNOMED <- sampleSNOMED()
 #'
 #' hasAttributes(c('Heart failure', 'Acute heart failure'),
